import arrow
import hashlib
import random
import json
from urllib.request import Request, urlopen
from sparweltbitool.logger import Logger
from etl_generic.etl import GenericException
import urllib
from base64 import b64encode


class ClientEmarsys(object):
    """
    Emarsys REST API wrapper.
    """

    def __init__(self,
                 username,
                 secret_token,
                 base_uri=u"https://www1.emarsys.net/api/v2/",
                 tzinfo_obj=None):
        """
        Initialises the Emarsys API wrapper object.
        """
        self._username = username
        self._secret_token = secret_token
        self._base_uri = base_uri
        self._tzinfo_obj = tzinfo_obj

    def __unicode__(self):
        return u"Emarsys({base_uri})".format(base_uri=self._base_uri)

    def __repr__(self):
        return unicode(self).encode("utf8")

    def call(self, uri, method, params=None):
        """
        Send the API call request to the Emarsys server.
        uri: API method URI
        method: HTTP method
        params: parameters to construct payload when API calls are made with
                POST or PUT HTTP methods.
        """
        uri = self._base_uri + uri
        if params:
            params = json.dumps(params).encode("utf-8")

        req = Request(uri, params)

        header_value = self._authentication_header_value()
        logger = Logger()
        logger.info(header_value)

        req.add_header("X-WSSE", header_value)
        req.add_header("Content-Type", "application/json")

        timeout = 600

        try:
            response = json.loads(urlopen(req, timeout=timeout).read().decode("utf-8"))

        except urllib.error.HTTPError as ex:
            message = "urllib.error.HTTPError on fetching for data from remote endpoint: '{}'. Code: {}, Reason: '{}'.".format(uri, ex.code, ex.reason)
            raise GenericException(message)
        except urllib.error.URLError as ex:
            message = "urllib.error.URLError on fetching for data from remote endpoint: '{}'. Reason: '{}'.".format(uri, ex.reason)
            raise GenericException(message)
        except UnicodeEncodeError as ex:
            raise GenericException(ex)
        except ValueError as ex:
            raise GenericException(ex)

        if response['replyText'] != 'OK' and response['replyCode'] != 0 and "data" in response:
            message='HTTP {reply_text}: {reply_code} [{uri}]'.format(
                reply_text=response['replyText'],
                reply_code=response['replyCode'],
                uri=uri,
            ),

            raise GenericException(message)

        return response["data"]

    def _authentication_header_value(self):
        created = arrow.utcnow().to('local').format('YYYY-MM-DDTHH:mm:ssZZ')
        nonce = hashlib.md5(str(random.getrandbits(128)).encode("utf-8")).hexdigest()
        password = "".join((nonce, created, self._secret_token))
        password_encoded_and_hashed = hashlib.sha1(password.encode("utf-8")).hexdigest()
        password_digest = b64encode(password_encoded_and_hashed.encode("utf-8")).decode("ascii")

        return ('UsernameToken Username="{username}", '
                'PasswordDigest="{password_digest}", Nonce="{nonce}", '
                'Created="{created}"').format(username=self._username,
                                              password_digest=password_digest,
                                              nonce=nonce,
                                              created=created)



from boto.s3.connection import S3Connection
import boto
from sparweltbitool.config import config
from sparweltbitool.logger import Logger

class ClientS3():
    """
    Operations with s3 bucket.
    Check documentation: http://boto.readthedocs.org/en/latest/ref/s3.html
    """

    def __init__(self):
        self.conn = S3Connection(config.get('aws', 'access_key_id'), config.get('aws', 'secret_access_key'))
        self.logger = Logger()

    def key_exists(self, key):
        """ Check if key exists."""
        conn = self.conn

        message = "Checking if key: '{}' exists on s3 bucket: '{}' set on region: '{}'".format(
            key,
            config.get('aws', 'bucket'),
            config.get('aws', 'region'))

        Logger().info(message)

        bucket = conn.get_bucket(config.get('aws', 'bucket'))
        if not bucket.get_location():
            conn = boto.s3.connect_to_region(config.get('aws', 'region'))
            bucket = conn.get_bucket(config.get('aws', 'bucket'))

        return bucket.get_key(key) is not None

    def key_delete(self, key):
        """ Check if key exists."""
        conn = self.conn

        message = "Deleting key: '{}' on s3 bucket: '{}' set on region: '{}'".format(
            key,
            config.get('aws', 'bucket'),
            config.get('aws', 'region'))

        Logger().info(message)

        bucket = conn.get_bucket(config.get('aws', 'bucket'))
        if not bucket.get_location():
            conn = boto.s3.connect_to_region(config.get('aws', 'region'))
            bucket = conn.get_bucket(config.get('aws', 'bucket'))

        return bucket.delete_key(key)

    def fetch_file_remote(self, key, file_path_local):
        """ Fetch key. Returns False if key not found and true if found."""
        conn = self.conn

        message = "Fetching to local file: '{}' from a key: '{}' on s3 bucket: '{}' set on region: '{}'".format(
            file_path_local,
            key,
            config.get('aws', 'bucket'),
            config.get('aws', 'region'))

        Logger().info(message)

        bucket = conn.get_bucket(config.get('aws', 'bucket'))
        if not bucket.get_location():
            conn = boto.s3.connect_to_region(config.get('aws', 'region'))
            bucket = conn.get_bucket(config.get('aws', 'bucket'))

        key_object = bucket.get_key(key)

        if key_object is not None:
            key_object.get_contents_to_filename(file_path_local)

            return True
        else:

            return False

    def send_file_local(self, key, file_path_local):
        """ Send key."""
        conn = self.conn

        message = "Sending local file: '{}' under a key: '{}' on s3 bucket: '{}' set on region: '{}'".format(
            file_path_local,
            key,
            config.get('aws', 'bucket'),
            config.get('aws', 'region'))

        Logger().debug(message)

        bucket = conn.get_bucket(config.get('aws', 'bucket'))
        if not bucket.get_location():
            conn = boto.s3.connect_to_region(config.get('aws', 'region'))
            bucket = conn.get_bucket(config.get('aws', 'bucket'))

        return bucket.new_key(key).set_contents_from_filename(file_path_local)

    def rename_files(self, prefix_old, prefix_new):
        """ Copy all keys from prefix_old to exact keys with prefix_new. Then delete original keys."""
        conn = self.conn

        bucket = conn.get_bucket(config.get('aws', 'bucket'))
        bucket_entries = bucket.list(prefix=prefix_old)

        count = 0

        for entry in bucket_entries:
            new_key_name = entry.name.replace(prefix_old, prefix_new)
            entry.copy(config.get('aws', 'bucket'), new_key_name)
            entry.delete()
            count += 1

        if count > 0:
            message = "{} files renamed from prefix: '{}' to new prefix: '{}' on s3 bucket: '{}' set on region: '{}'".format(
                count,
                prefix_old,
                prefix_new,
                config.get('aws', 'bucket'),
                config.get('aws', 'region'))
        else:
            message = "Tried to rename files from prefix: '{}' to new prefix: '{}' on s3 bucket: '{}' set on region: '{}' but none found.".format(
                prefix_old,
                prefix_new,
                config.get('aws', 'bucket'),
                config.get('aws', 'region'))

        Logger().debug(message)



import redis

from sparweltbitool.config import config
from sparweltbitool.singleton import Singleton


@Singleton
class ClientRedis(object):
    """ Operations with Redis."""

    def __init__(self):

        self.r = redis.StrictRedis(
            host=config.get('redis', 'host'),
            port=config.get('redis', 'port'),
            db=config.get('redis', 'db'))



import os
import sys
if sys.version_info[:2] >= (3, 4):
    import configparser
    config = configparser.ConfigParser()
else:
    import ConfigParser
    config = ConfigParser.ConfigParser()

config.readfp(open('app/config/config_%s.cfg' % os.environ.get('APP_ENV', 'dev')))



from voluptuous import Invalid
import re

def Url(msg=None):
    def f(v):

        regex = re.compile(
            r'^(?:http|ftp)s?://' # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
            r'localhost|' #localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
            r'(?::\d+)?' # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)

        if re.match(regex, str(v)):
            return str(v)
        else:
            raise Invalid(msg or "value is not correct Uniform Resource Locator")
    return f

def TrackableCid(msg=None):
    def f(v):
        regex = re.compile(r'^[A-Z]{2}-[A-Z]{2}-[1-9]{1}[0-9]*$', re.IGNORECASE)
        if re.match(regex, str(v)):
            return str(v)
        else:
            raise Invalid(msg or "value is not correct trackable CID")
    return f
